[![FactSet](https://raw.githubusercontent.com/factset/enterprise-sdk/main/docs/images/factset-logo.svg)](https://www.factset.com)

# FactSet Estimates client library for Python

[![API Version](https://img.shields.io/badge/api-v2.6.0-blue)]()
[![PyPi](https://img.shields.io/pypi/v/fds.sdk.FactSetEstimates)](https://pypi.org/project/fds.sdk.FactSetEstimates/)
[![Apache-2 license](https://img.shields.io/badge/license-Apache2-brightgreen.svg)](https://www.apache.org/licenses/LICENSE-2.0)

With global coverage since 1999, the FactSet Estimates API provides you with comprehensive estimates and statistics on a wide variety of financial statement items as well as industry-specific metrics. The universe is comprised of over 19,000 active companies across 90+ countries with the following types of data included:
 
- **Consensus**
- **Detail**
- **Ratings**
- **Surprise**
- **Segments**
- **New Estimates and Ratings Reports Endpoints**

For clients seeking  curated and relevant financial data, the FactSet Estimates API now includes Estimates and Ratings Reports endpoints. These powerful endpoints are designed for easy integration and consumption, delivering a wide array of financial metrics, estimates, and critical statistics in a highly accessible format suitable for both mobile and web applications.
 
Whether you are an analyst, investor, or financial professional, the Estimates and Ratings Reports endpoints offer detailed and actionable financial insights that can support thorough analyses and strategic decision-making processes.
 
The Estimates and Ratings Reports endpoints are especially valuable for B2B2C  applications, empowering financial services firms, investment companies, and corporate finance teams to:
 
- **Elevate Client Engagement:** Enrich user experiences in client-facing applications with comprehensive and up-to-date financial metrics.

- **Build Custom Reporting Tools:** Create tailored dashboards and analytics tools that provide deep insights and foster better financial understanding.

By adopting the FactSet Estimates API with its enriched Estimates and Ratings Reports endpoints, businesses can streamline their financial data integration process, improve operational efficiency, and deliver superior financial insights to their clients and end-users.
    <p>This API is rate-limited to 10 requests per second and 10 concurrent requests per user.</p>

**Download API Specification**
    
  To programmatically download the FactSet Estimates API Specification file in .yaml format, utilize the link below. You must be authorized for this API to extract the specification. This specification can then be used for Codegen to create your own SDKs. You can also access it by selecting the \"Download Spec\" button beside the version information.
  
  [https://api.factset.com/content/factset-estimates/v2/spec/swagger.yaml](https://api.factset.com/content/factset-estimates/v2/spec/swagger.yaml)


This Python package is automatically generated by the [OpenAPI Generator](https://openapi-generator.tech) project:

- API version: 2.6.0
- SDK version: 1.2.0
- Build package: org.openapitools.codegen.languages.PythonClientCodegen

## Requirements

* Python >= 3.7

## Installation

### Poetry

```shell
poetry add fds.sdk.utils fds.sdk.FactSetEstimates==1.2.0
```

### pip

```shell
pip install fds.sdk.utils fds.sdk.FactSetEstimates==1.2.0
```

## Usage

1. [Generate authentication credentials](../../../../README.md#authentication).
2. Setup Python environment.
   1. Install and activate python 3.7+. If you're using [pyenv](https://github.com/pyenv/pyenv):

      ```sh
      pyenv install 3.9.7
      pyenv shell 3.9.7
      ```

   2. (optional) [Install poetry](https://python-poetry.org/docs/#installation).
3. [Install dependencies](#installation).
4. Run the following:

> [!IMPORTANT]
> The parameter variables defined below are just examples and may potentially contain non valid values. Please replace them with valid values.

### Example Code

```python
from fds.sdk.utils.authentication import ConfidentialClient

import fds.sdk.FactSetEstimates
from fds.sdk.FactSetEstimates.api import broker_detail_api
from fds.sdk.FactSetEstimates.models import *
from dateutil.parser import parse as dateutil_parser
from pprint import pprint

# See configuration.py for a list of all supported configuration parameters.

# Examples for each supported authentication method are below,
# choose one that satisfies your use case.

# (Preferred) OAuth 2.0: FactSetOAuth2
# See https://github.com/FactSet/enterprise-sdk#oauth-20
# for information on how to create the app-config.json file
#
# The confidential client instance should be reused in production environments.
# See https://github.com/FactSet/enterprise-sdk-utils-python#authentication
# for more information on using the ConfidentialClient class
configuration = fds.sdk.FactSetEstimates.Configuration(
    fds_oauth_client=ConfidentialClient('/path/to/app-config.json')
)

# Basic authentication: FactSetApiKey
# See https://github.com/FactSet/enterprise-sdk#api-key
# for information how to create an API key
# configuration = fds.sdk.FactSetEstimates.Configuration(
#     username='USERNAME-SERIAL',
#     password='API-KEY'
# )

# Enter a context with an instance of the API client
with fds.sdk.FactSetEstimates.ApiClient(configuration) as api_client:
    # Create an instance of the API class
    api_instance = broker_detail_api.BrokerDetailApi(api_client)
    ids = ["AAPL-USA"] # [str] | Security or Entity identifiers. FactSet Identifiers, tickers, CUSIP and SEDOL are accepted input. <p>***ids limit** =  3000 per request*</p> * Make Note - id limit of 3000 for defaults, otherwise the service is limited to a 30 second duration. This can be reached when increasing total number of metrics requested and depth of history. * 
    metrics = ["SALES"] # [str] | Requested metrics. Use the /metrics endpoint to return a list of available estimate items. Note, the number of metrics you are allowed to supply is limited to 1 for now. **Top 10** most used metrics are **EPS, SALES, DPS, EBITDA,EBIT, PRICE_TGT, CFPS, BPS, NET_INC, and ASSETS**.  For more details, visit [Online Assistant Page #15034](https://oa.apps.factset.com/pages/15034). 
    start_date = "2019-07-30" # str | Start date for point in time of estimates expressed in YYYY-MM-DD format. (optional)
    end_date = "2019-08-30" # str | End date for point in time of estimates expressed in YYYY-MM-DD format. (optional)
    frequency = "D" # str | Controls the frequency of the data returned.   * **D** = Daily   * **W** = Weekly, based on the last day of the week of the start date.   * **AM** = Monthly, based on the start date (e.g., if the start date is June 16, data is displayed for June 16, May 16, April 16 etc.).         * **AQ** = Quarterly, based on the start date.   * **AY** = Actual Annual, based on the start date.   (optional) if omitted the server will use the default value of "D"
    periodicity = "ANN" # str | The periodicity for the estimates requested, allowing you to fetch Quarterly, Semi-Annual, and Annual Estimates.   * **ANN** - Annual   * **QTR** - Quarterly   * **SEMI** - Semi-Annual   (optional) if omitted the server will use the default value of "ANN"
    include_all = False # bool | Include All filter is used to identify included and excluded broker details from the consensus   By default the service would return only the brokers included in the consensus-   * **TRUE** = Returns all the brokers included and excluded in the consensus   * **FALSE** = Returns only the broker details included in the consensus     (optional) if omitted the server will use the default value of False
    fiscal_period_start = "2019" # str | Fiscal period start expressed in absolute date formats. Date that will fall back to most recent completed period during resolution.   * **Fiscal Quarter-end** - YYYY/FQ (e.g., 2019/1F, 2019/2F, 2019/3F, 2019/4F)   * **Fiscal Year-end** - YYYY (e.g. 2019)  (optional)
    fiscal_period_end = "2020" # str | Fiscal period start expressed in absolute date formats. Date that will fall back to most recent completed period during resolution.   * **Fiscal Quarter-end** - YYYY/FQ (e.g., 2019/1F, 2019/2F, 2019/3F, 2019/4F)   * **Fiscal Year-end** - YYYY (e.g. 2019)  (optional)
    currency = "USD" # str | Currency code for adjusting the data. Use 'ESTIMATE' as input value for the values in Estimate Currency. For a list of currency ISO codes, visit [Online Assistant Page #1470](https://oa.apps.factset.com/pages/1470). (optional)

    try:
        # Estimates detail data for fixed fiscal periods
        # example passing only required values which don't have defaults set
        # and optional values
        api_response = api_instance.get_fixed_detail(ids, metrics, start_date=start_date, end_date=end_date, frequency=frequency, periodicity=periodicity, include_all=include_all, fiscal_period_start=fiscal_period_start, fiscal_period_end=fiscal_period_end, currency=currency)

        pprint(api_response)
    except fds.sdk.FactSetEstimates.ApiException as e:
        print("Exception when calling BrokerDetailApi->get_fixed_detail: %s\n" % e)

    # # Get response, http status code and response headers
    # try:
    #     # Estimates detail data for fixed fiscal periods
    #     api_response, http_status_code, response_headers = api_instance.get_fixed_detail_with_http_info(ids, metrics, start_date=start_date, end_date=end_date, frequency=frequency, periodicity=periodicity, include_all=include_all, fiscal_period_start=fiscal_period_start, fiscal_period_end=fiscal_period_end, currency=currency)


    #     pprint(api_response)
    #     pprint(http_status_code)
    #     pprint(response_headers)
    # except fds.sdk.FactSetEstimates.ApiException as e:
    #     print("Exception when calling BrokerDetailApi->get_fixed_detail: %s\n" % e)

    # # Get response asynchronous
    # try:
    #     # Estimates detail data for fixed fiscal periods
    #     async_result = api_instance.get_fixed_detail_async(ids, metrics, start_date=start_date, end_date=end_date, frequency=frequency, periodicity=periodicity, include_all=include_all, fiscal_period_start=fiscal_period_start, fiscal_period_end=fiscal_period_end, currency=currency)
    #     api_response = async_result.get()


    #     pprint(api_response)
    # except fds.sdk.FactSetEstimates.ApiException as e:
    #     print("Exception when calling BrokerDetailApi->get_fixed_detail: %s\n" % e)

    # # Get response, http status code and response headers asynchronous
    # try:
    #     # Estimates detail data for fixed fiscal periods
    #     async_result = api_instance.get_fixed_detail_with_http_info_async(ids, metrics, start_date=start_date, end_date=end_date, frequency=frequency, periodicity=periodicity, include_all=include_all, fiscal_period_start=fiscal_period_start, fiscal_period_end=fiscal_period_end, currency=currency)
    #     api_response, http_status_code, response_headers = async_result.get()


    #     pprint(api_response)
    #     pprint(http_status_code)
    #     pprint(response_headers)
    # except fds.sdk.FactSetEstimates.ApiException as e:
    #     print("Exception when calling BrokerDetailApi->get_fixed_detail: %s\n" % e)

```

### Using Pandas

To convert an API response to a Pandas DataFrame, it is necessary to transform it first to a dictionary.
```python
import pandas as pd

response_dict = api_response.to_dict()['data']

simple_json_response = pd.DataFrame(response_dict)
nested_json_response = pd.json_normalize(response_dict)
```

### Debugging

The SDK uses the standard library [`logging`](https://docs.python.org/3/library/logging.html#module-logging) module.

Setting `debug` to `True` on an instance of the `Configuration` class sets the log-level of related packages to `DEBUG`
and enables additional logging in Pythons [HTTP Client](https://docs.python.org/3/library/http.client.html).

**Note**: This prints out sensitive information (e.g. the full request and response). Use with care.

```python
import logging
import fds.sdk.FactSetEstimates

logging.basicConfig(level=logging.DEBUG)

configuration = fds.sdk.FactSetEstimates.Configuration(...)
configuration.debug = True
```

### Configure a Proxy

You can pass proxy settings to the Configuration class:

* `proxy`: The URL of the proxy to use.
* `proxy_headers`: a dictionary to pass additional headers to the proxy (e.g. `Proxy-Authorization`).

```python
import fds.sdk.FactSetEstimates

configuration = fds.sdk.FactSetEstimates.Configuration(
    # ...
    proxy="http://secret:password@localhost:5050",
    proxy_headers={
        "Custom-Proxy-Header": "Custom-Proxy-Header-Value"
    }
)
```

### Custom SSL Certificate

TLS/SSL certificate verification can be configured with the following Configuration parameters:

* `ssl_ca_cert`: a path to the certificate to use for verification in `PEM` format.
* `verify_ssl`: setting this to `False` disables the verification of certificates.
  Disabling the verification is not recommended, but it might be useful during
  local development or testing.

```python
import fds.sdk.FactSetEstimates

configuration = fds.sdk.FactSetEstimates.Configuration(
    # ...
    ssl_ca_cert='/path/to/ca.pem'
)
```

### Request Retries

In case the request retry behaviour should be customized, it is possible to pass a `urllib3.Retry` object to the `retry` property of the Configuration.

```python
from urllib3 import Retry
import fds.sdk.FactSetEstimates

configuration = fds.sdk.FactSetEstimates.Configuration(
    # ...
)

configuration.retries = Retry(total=3, status_forcelist=[500, 502, 503, 504])
```


## Documentation for API Endpoints

All URIs are relative to *https://api.factset.com/content*

Class | Method | HTTP request | Description
------------ | ------------- | ------------- | -------------
*BrokerDetailApi* | [**get_fixed_detail**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/BrokerDetailApi.md#get_fixed_detail) | **GET** /factset-estimates/v2/fixed-detail | Estimates detail data for fixed fiscal periods
*BrokerDetailApi* | [**get_fixed_detail_for_list**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/BrokerDetailApi.md#get_fixed_detail_for_list) | **POST** /factset-estimates/v2/fixed-detail | Estimates detail data for fixed fiscal periods
*BrokerDetailApi* | [**get_rolling_detail**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/BrokerDetailApi.md#get_rolling_detail) | **GET** /factset-estimates/v2/rolling-detail | FactSet estimates detail data for rolling fiscal periods
*BrokerDetailApi* | [**get_rolling_detail_for_list**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/BrokerDetailApi.md#get_rolling_detail_for_list) | **POST** /factset-estimates/v2/rolling-detail | FactSet estimates detail data for rolling fiscal periods
*ConsensusApi* | [**get_fixed_consensus**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/ConsensusApi.md#get_fixed_consensus) | **GET** /factset-estimates/v2/fixed-consensus | Retrieves consensus estimates for a requested list of ids and fixed fiscal periods
*ConsensusApi* | [**get_fixed_consensus_for_list**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/ConsensusApi.md#get_fixed_consensus_for_list) | **POST** /factset-estimates/v2/fixed-consensus | FactSet consensus estimates for fixed fiscal periods
*ConsensusApi* | [**get_rolling_consensus**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/ConsensusApi.md#get_rolling_consensus) | **GET** /factset-estimates/v2/rolling-consensus | Retrieves consensus estimates for a requested list of ids and rolling fiscal periods.
*ConsensusApi* | [**get_rolling_consensus_for_list**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/ConsensusApi.md#get_rolling_consensus_for_list) | **POST** /factset-estimates/v2/rolling-consensus | Retrieves consensus estimates for a requested list of ids and rolling fiscal periods
*DataItemsApi* | [**get_estimate_metrics**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/DataItemsApi.md#get_estimate_metrics) | **GET** /factset-estimates/v2/metrics | Available Estimate metrics
*DataItemsApi* | [**get_estimate_metrics_for_list**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/DataItemsApi.md#get_estimate_metrics_for_list) | **POST** /factset-estimates/v2/metrics | Available Estimate metrics or ratios.
*EstimatesAndRatingsReportsApi* | [**get_estimate_types**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/EstimatesAndRatingsReportsApi.md#get_estimate_types) | **GET** /factset-estimates/v2/company-reports/estimate-types | Returns a list of valid estimate types.
*EstimatesAndRatingsReportsApi* | [**get_estimates**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/EstimatesAndRatingsReportsApi.md#get_estimates) | **GET** /factset-estimates/v2/company-reports/forecast | Returns forecasted estimates.
*EstimatesAndRatingsReportsApi* | [**get_surprise_history**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/EstimatesAndRatingsReportsApi.md#get_surprise_history) | **GET** /factset-estimates/v2/company-reports/surprise-history | Surprise History
*RatingsApi* | [**get_consensus_ratings**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/RatingsApi.md#get_consensus_ratings) | **GET** /factset-estimates/v2/consensus-ratings | Ratings consensus estimates to fetch Buy, overWeight, Hold, underWeight, and Sell.
*RatingsApi* | [**get_consensus_ratings_for_list**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/RatingsApi.md#get_consensus_ratings_for_list) | **POST** /factset-estimates/v2/consensus-ratings | Ratings consensus estimates to fetch Buy, overWeight, Hold, underWeight, and Sell.
*RatingsApi* | [**get_detail_ratings**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/RatingsApi.md#get_detail_ratings) | **GET** /factset-estimates/v2/detail-ratings | Broker Detail estimates to fetch Buy, overWeight, Hold, underWeight, and Sell.
*RatingsApi* | [**get_detail_ratings_for_list**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/RatingsApi.md#get_detail_ratings_for_list) | **POST** /factset-estimates/v2/detail-ratings | Broker Detail estimates to fetch Buy, overWeight, Hold, underWeight, and Sell.
*SegmentsApi* | [**get_segments**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/SegmentsApi.md#get_segments) | **GET** /factset-estimates/v2/segments | Retrieves product &amp; geographic segment estimates for a requested list of ids and fiscal periods
*SegmentsApi* | [**get_segments_for_list**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/SegmentsApi.md#get_segments_for_list) | **POST** /factset-estimates/v2/segments | Retrieves product segment estimates for a requested list of ids and fiscal periods
*SurpriseApi* | [**get_surprise**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/SurpriseApi.md#get_surprise) | **GET** /factset-estimates/v2/surprise | Surprise estimates for rolling fiscal periods
*SurpriseApi* | [**get_surprise_for_list**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/SurpriseApi.md#get_surprise_for_list) | **POST** /factset-estimates/v2/surprise | Surprise estimates for rolling fiscal periods


## Documentation For Models

 - [AnalystRating](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/AnalystRating.md)
 - [AnalystRatingResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/AnalystRatingResponse.md)
 - [Category](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/Category.md)
 - [ConsensusEstimate](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/ConsensusEstimate.md)
 - [ConsensusRatings](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/ConsensusRatings.md)
 - [ConsensusRatingsRequest](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/ConsensusRatingsRequest.md)
 - [ConsensusRatingsResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/ConsensusRatingsResponse.md)
 - [ConsensusResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/ConsensusResponse.md)
 - [DetailEstimate](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/DetailEstimate.md)
 - [DetailRatings](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/DetailRatings.md)
 - [DetailRatingsRequest](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/DetailRatingsRequest.md)
 - [DetailRatingsResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/DetailRatingsResponse.md)
 - [DetailResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/DetailResponse.md)
 - [ErrorObject](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/ErrorObject.md)
 - [ErrorResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/ErrorResponse.md)
 - [ErrorResponseSubErrors](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/ErrorResponseSubErrors.md)
 - [Estimate](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/Estimate.md)
 - [EstimateResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/EstimateResponse.md)
 - [EstimateResponseData](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/EstimateResponseData.md)
 - [EstimateType](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/EstimateType.md)
 - [EstimateTypesResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/EstimateTypesResponse.md)
 - [FixedConsensusRequest](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/FixedConsensusRequest.md)
 - [FixedDetailRequest](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/FixedDetailRequest.md)
 - [Frequency](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/Frequency.md)
 - [Ids](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/Ids.md)
 - [Meta](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/Meta.md)
 - [Metric](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/Metric.md)
 - [Metrics](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/Metrics.md)
 - [MetricsRequest](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/MetricsRequest.md)
 - [MetricsResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/MetricsResponse.md)
 - [Pagination](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/Pagination.md)
 - [Periodicity](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/Periodicity.md)
 - [PeriodicityDetail](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/PeriodicityDetail.md)
 - [PeriodicitySurprise](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/PeriodicitySurprise.md)
 - [RatingsCount](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/RatingsCount.md)
 - [RollingConsensusRequest](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/RollingConsensusRequest.md)
 - [RollingDetailRequest](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/RollingDetailRequest.md)
 - [SegmentIds](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/SegmentIds.md)
 - [SegmentType](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/SegmentType.md)
 - [SegmentsEstimate](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/SegmentsEstimate.md)
 - [SegmentsRequest](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/SegmentsRequest.md)
 - [SegmentsResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/SegmentsResponse.md)
 - [Statistic](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/Statistic.md)
 - [Subcategory](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/Subcategory.md)
 - [Surprise](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/Surprise.md)
 - [SurpriseHistory](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/SurpriseHistory.md)
 - [SurpriseHistoryResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/SurpriseHistoryResponse.md)
 - [SurpriseHistoryResponseData](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/SurpriseHistoryResponseData.md)
 - [SurpriseRequest](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/SurpriseRequest.md)
 - [SurpriseResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/SurpriseResponse.md)
 - [TargetPrice](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetEstimates/v2/docs/TargetPrice.md)


## Documentation For Authorization


## FactSetApiKey

- **Type**: HTTP basic authentication


## FactSetOAuth2

- **Type**: OAuth
- **Flow**: application
- **Authorization URL**: 
- **Scopes**: N/A


## Notes for Large OpenAPI documents
If the OpenAPI document is large, imports in fds.sdk.FactSetEstimates.apis and fds.sdk.FactSetEstimates.models may fail with a
RecursionError indicating the maximum recursion limit has been exceeded. In that case, there are a couple of solutions:

Solution 1:
Use specific imports for apis and models like:
- `from fds.sdk.FactSetEstimates.api.default_api import DefaultApi`
- `from fds.sdk.FactSetEstimates.model.pet import Pet`

Solution 2:
Before importing the package, adjust the maximum recursion limit as shown below:
```
import sys
sys.setrecursionlimit(1500)
import fds.sdk.FactSetEstimates
from fds.sdk.FactSetEstimates.apis import *
from fds.sdk.FactSetEstimates.models import *
```

## Contributing

Please refer to the [contributing guide](../../../../CONTRIBUTING.md).

## Copyright

Copyright 2022 FactSet Research Systems Inc

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

